unit uSalesman;

interface

uses
  uHouse                        // for THouse, TWindow, TDoor
  ;

type

  TDoubleGlazingSalesman = class(THouseVisitor)
  private

    FSomethingIsWood: boolean;        // will be initialised to false

  public

    procedure AllowVisitorToDoSomethingTo(House: THouse); override;
    procedure AllowVisitorToDoSomethingTo(Window: TWindow); override;
    procedure AllowVisitorToDoSomethingTo(Door: TDoor); override;
    property SomethingIsWood: boolean read FSomethingIsWood;

  end;

procedure SellToHouse(House: THouse);

implementation

uses
  dialogs                       // for MessageDlg
  , uInstaller                  // for TDoubleGlazingInstaller
  , Controls                    // for mrYes
  ;

procedure SellToHouse(House: THouse);
var
  Visitor: TDoubleGlazingSalesman;
begin
  Visitor := TDoubleGlazingSalesman.Create;
  try { ... finally }
    House.AcceptVisitFrom(Visitor);
    if Visitor.SomethingIsWood then
    begin
      if (MessageDlg('Do you want to replace your wooden bits with plastic?',
                    mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
      begin
        uInstaller.InstallPlasticInHouse(House);
      end;
    end;
  finally
    Visitor.Free;
  end;  { try ... finally }
end;

{ TDoubleGlazingSalesman }

procedure TDoubleGlazingSalesman.AllowVisitorToDoSomethingTo(
  House: THouse);
begin
  // Nothing to do to a house
end;

procedure TDoubleGlazingSalesman.AllowVisitorToDoSomethingTo(
  Window: TWindow);
begin
  FSomethingIsWood := Window.Material = tmWood;
end;

procedure TDoubleGlazingSalesman.AllowVisitorToDoSomethingTo(Door: TDoor);
begin
  FSomethingIsWood := Door.Material = tmWood;
end;

end.
